/*********************************************************************
*                SEGGER MICROCONTROLLER GmbH & Co. KG                *
*        Solutions for real time microcontroller applications        *
**********************************************************************
*                                                                    *
*        (c) 2003-2014     SEGGER Microcontroller GmbH & Co KG       *
*                                                                    *
*        Internet: www.segger.com    Support:  support@segger.com    *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File        : TM4C129.c
Purpose     : Hardware specific routines
---------------------------END-OF-HEADER------------------------------
*/

#include "GUI.h"

#include "HW_TM4C129.h"
#include "rom.h"
#include "touch.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/
#define SYS_CLK 120000000L

/*********************************************************************
*
*       Static code
*
**********************************************************************
*/
/*********************************************************************
*
*       _GPIODirModeSet
*/
static void _GPIODirModeSet(uint32_t ui32Port, uint8_t ui8Pins, uint32_t ui32PinIO) {
    //
    // Check the arguments.
    //
    ASSERT(_GPIOBaseValid(ui32Port));
    ASSERT((ui32PinIO == GPIO_DIR_MODE_IN) ||
           (ui32PinIO == GPIO_DIR_MODE_OUT) ||
           (ui32PinIO == GPIO_DIR_MODE_HW));

    //
    // Set the pin direction and mode.
    //
    HWREG(ui32Port + GPIO_O_DIR) = ((ui32PinIO & 1) ?
                                    (HWREG(ui32Port + GPIO_O_DIR) | ui8Pins) :
                                    (HWREG(ui32Port + GPIO_O_DIR) & ~(ui8Pins)));
    HWREG(ui32Port + GPIO_O_AFSEL) = ((ui32PinIO & 2) ?
                                      (HWREG(ui32Port + GPIO_O_AFSEL) |
                                       ui8Pins) :
                                      (HWREG(ui32Port + GPIO_O_AFSEL) &
                                       ~(ui8Pins)));
}

/*********************************************************************
*
*       _GPIOPadConfigSet
*/
static void _GPIOPadConfigSet(uint32_t ui32Port, uint8_t ui8Pins, uint32_t ui32Strength, uint32_t ui32PinType) {
    uint8_t ui8Bit;

    //
    // Check the arguments.
    //
    ASSERT(_GPIOBaseValid(ui32Port));
    ASSERT((ui32Strength == GPIO_STRENGTH_2MA) ||
           (ui32Strength == GPIO_STRENGTH_4MA) ||
           (ui32Strength == GPIO_STRENGTH_6MA) ||
           (ui32Strength == GPIO_STRENGTH_8MA) ||
           (ui32Strength == GPIO_STRENGTH_8MA_SC) ||
           (ui32Strength == GPIO_STRENGTH_10MA) ||
           (ui32Strength == GPIO_STRENGTH_12MA));
    ASSERT((ui32PinType == GPIO_PIN_TYPE_STD) ||
           (ui32PinType == GPIO_PIN_TYPE_STD_WPU) ||
           (ui32PinType == GPIO_PIN_TYPE_STD_WPD) ||
           (ui32PinType == GPIO_PIN_TYPE_OD) ||
           (ui32PinType == GPIO_PIN_TYPE_WAKE_LOW) ||
           (ui32PinType == GPIO_PIN_TYPE_WAKE_HIGH) ||
           (ui32PinType == GPIO_PIN_TYPE_ANALOG));

    //
    // Set the output drive strength.
    //
    HWREG(ui32Port + GPIO_O_DR2R) = ((ui32Strength & 1) ?
                                     (HWREG(ui32Port + GPIO_O_DR2R) |
                                      ui8Pins) :
                                     (HWREG(ui32Port + GPIO_O_DR2R) &
                                      ~(ui8Pins)));
    HWREG(ui32Port + GPIO_O_DR4R) = ((ui32Strength & 2) ?
                                     (HWREG(ui32Port + GPIO_O_DR4R) |
                                      ui8Pins) :
                                     (HWREG(ui32Port + GPIO_O_DR4R) &
                                      ~(ui8Pins)));
    HWREG(ui32Port + GPIO_O_DR8R) = ((ui32Strength & 4) ?
                                     (HWREG(ui32Port + GPIO_O_DR8R) |
                                      ui8Pins) :
                                     (HWREG(ui32Port + GPIO_O_DR8R) &
                                      ~(ui8Pins)));
    HWREG(ui32Port + GPIO_O_SLR) = ((ui32Strength & 8) ?
                                    (HWREG(ui32Port + GPIO_O_SLR) |
                                     ui8Pins) :
                                    (HWREG(ui32Port + GPIO_O_SLR) &
                                     ~(ui8Pins)));

    //
    // Set the 12-mA drive select register and the GPIO peripheral
    // configuration register.  These registers only appear in Flurry-class
    // (and later) devices, but are harmless writes on older devices.
    //
    HWREG(ui32Port + GPIO_O_DR12R) = ((ui32Strength & 0x10) ?
                                      (HWREG(ui32Port + GPIO_O_DR12R) |
                                       ui8Pins) :
                                      (HWREG(ui32Port + GPIO_O_DR12R) &
                                       ~(ui8Pins)));
    //
    // Walk pins 0-7 and clear or set the provided PC[EDMn] encoding.
    //
    for(ui8Bit = 0; ui8Bit < 8; ui8Bit++)
    {
        if(ui8Pins & (1 << ui8Bit))
        {
            HWREG(ui32Port + GPIO_O_PC) = (HWREG(ui32Port + GPIO_O_PC) &
                                           ~(0x3 << (2 * ui8Bit)));
            HWREG(ui32Port + GPIO_O_PC) |= (((ui32Strength >> 5) & 0x3) <<
                                            (2 * ui8Bit));
        }
    }

    //
    // Set the pin type.
    //
    HWREG(ui32Port + GPIO_O_ODR) = ((ui32PinType & 1) ?
                                    (HWREG(ui32Port + GPIO_O_ODR) | ui8Pins) :
                                    (HWREG(ui32Port + GPIO_O_ODR) & ~(ui8Pins)));
    HWREG(ui32Port + GPIO_O_PUR) = ((ui32PinType & 2) ?
                                    (HWREG(ui32Port + GPIO_O_PUR) | ui8Pins) :
                                    (HWREG(ui32Port + GPIO_O_PUR) & ~(ui8Pins)));
    HWREG(ui32Port + GPIO_O_PDR) = ((ui32PinType & 4) ?
                                    (HWREG(ui32Port + GPIO_O_PDR) | ui8Pins) :
                                    (HWREG(ui32Port + GPIO_O_PDR) & ~(ui8Pins)));
    HWREG(ui32Port + GPIO_O_DEN) = ((ui32PinType & 8) ?
                                    (HWREG(ui32Port + GPIO_O_DEN) | ui8Pins) :
                                    (HWREG(ui32Port + GPIO_O_DEN) & ~(ui8Pins)));

    //
    // Set the wake pin enable register and the wake level register.
    // These registers only appear in Flurry-class (and later) devices, but
    // are harmless writes on older devices.
    //
    HWREG(ui32Port + GPIO_O_WAKELVL) = ((ui32PinType & 0x200) ?
                                        (HWREG(ui32Port + GPIO_O_WAKELVL) |
                                         ui8Pins) :
                                        (HWREG(ui32Port + GPIO_O_WAKELVL) &
                                         ~(ui8Pins)));
    HWREG(ui32Port + GPIO_O_WAKEPEN) = ((ui32PinType & 0x300) ?
                                        (HWREG(ui32Port + GPIO_O_WAKEPEN) |
                                         ui8Pins) :
                                        (HWREG(ui32Port + GPIO_O_WAKEPEN) &
                                         ~(ui8Pins)));

    //
    // Set the analog mode select register.
    //
    HWREG(ui32Port + GPIO_O_AMSEL) =
        ((ui32PinType == GPIO_PIN_TYPE_ANALOG) ?
         (HWREG(ui32Port + GPIO_O_AMSEL) | ui8Pins) :
         (HWREG(ui32Port + GPIO_O_AMSEL) & ~(ui8Pins)));
}

/*********************************************************************
*
*       _GPIOPinTypeEthernetLED
*/
static void _GPIOPinTypeEthernetLED(uint32_t ui32Port, uint8_t ui8Pins) {
    //
    // Check the arguments.
    //
    ASSERT(_GPIOBaseValid(ui32Port));

    //
    // Make the pin(s) be peripheral controlled.
    //
    _GPIODirModeSet(ui32Port, ui8Pins, GPIO_DIR_MODE_HW);

    //
    // Set the pad(s) for standard push-pull operation.
    //
    _GPIOPadConfigSet(ui32Port, ui8Pins, GPIO_STRENGTH_8MA, GPIO_PIN_TYPE_STD);
}

/*********************************************************************
*
*       _Pinout
*/
static void _Pinout(void) {
  //
  // Enable all the GPIO peripherals.
  //
  ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOA);
  ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOB);
  ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOC);
  ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOD);
  ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOE);
  ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOF);
  ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOG);
  ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOH);
  ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOJ);
  ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOK);
  ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOL);
  ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOM);
  ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_GPION);
  ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOP);
  ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOQ);
  ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOR);
  ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOS);
  ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOT);

  //
  // PA0-1 are used for UART0.
  //
  ROM_GPIOPinConfigure(GPIO_PA0_U0RX);
  ROM_GPIOPinConfigure(GPIO_PA1_U0TX);
  ROM_GPIOPinTypeUART(GPIO_PORTA_BASE, GPIO_PIN_0 | GPIO_PIN_1);

  //
  // PA2-5 are used for SSI0 to the second booster pack.
  //
  ROM_GPIOPinConfigure(GPIO_PA2_SSI0CLK);
  ROM_GPIOPinConfigure(GPIO_PA3_SSI0FSS);
  ROM_GPIOPinConfigure(GPIO_PA4_SSI0XDAT0);
  ROM_GPIOPinConfigure(GPIO_PA5_SSI0XDAT1);

  //
  // PB0-1/PD6-7/PL6-7 are used for USB.
  //
  HWREG(GPIO_PORTD_BASE + GPIO_O_LOCK) = GPIO_LOCK_KEY;
  HWREG(GPIO_PORTD_BASE + GPIO_O_CR) = 0xff;
  ROM_GPIOPinConfigure(GPIO_PD6_USB0EPEN);
  ROM_GPIOPinConfigure(GPIO_PD7_USB0PFLT);
  ROM_GPIOPinTypeUSBAnalog(GPIO_PORTB_BASE, GPIO_PIN_0 | GPIO_PIN_1);
  ROM_GPIOPinTypeUSBDigital(GPIO_PORTD_BASE, GPIO_PIN_6 | GPIO_PIN_7);
  ROM_GPIOPinTypeUSBAnalog(GPIO_PORTL_BASE, GPIO_PIN_6 | GPIO_PIN_7);

  //
  // PB2/PD4 are used for the speaker output.
  //
  ROM_GPIOPinConfigure(GPIO_PB2_T5CCP0);
  ROM_GPIOPinTypeTimer(GPIO_PORTB_BASE, GPIO_PIN_2);
  ROM_GPIOPinTypeGPIOOutput(GPIO_PORTD_BASE, GPIO_PIN_4);
  ROM_GPIOPinWrite(GPIO_PORTD_BASE, GPIO_PIN_4, 0);

  //
  // PB6-7 are used for I2C to the TMP100 and the EM connector.
  //
  ROM_GPIOPinConfigure(GPIO_PB6_I2C6SCL);
  ROM_GPIOPinConfigure(GPIO_PB7_I2C6SDA);
  ROM_GPIOPinTypeI2CSCL(GPIO_PORTB_BASE, GPIO_PIN_6);
  ROM_GPIOPinTypeI2C(GPIO_PORTB_BASE, GPIO_PIN_7);

  //
  // PE5/PN3/PP1 are used for the push buttons.
  //
  ROM_GPIOPinTypeGPIOInput(GPIO_PORTE_BASE, GPIO_PIN_5);
  ROM_GPIOPinTypeGPIOInput(GPIO_PORTN_BASE, GPIO_PIN_3);
  ROM_GPIOPinTypeGPIOInput(GPIO_PORTP_BASE, GPIO_PIN_1);

  //
  // PE7/PP7/PT2-3 are used for the touch screen.
  //
  HWREG(GPIO_PORTE_BASE + GPIO_O_LOCK) = GPIO_LOCK_KEY;
  HWREG(GPIO_PORTE_BASE + GPIO_O_CR) = 0xff;
  ROM_GPIOPinTypeGPIOInput(GPIO_PORTE_BASE, GPIO_PIN_7);
  ROM_GPIOPinTypeGPIOInput(GPIO_PORTP_BASE, GPIO_PIN_7);
  ROM_GPIOPinTypeGPIOInput(GPIO_PORTT_BASE, GPIO_PIN_2 | GPIO_PIN_3);
  ROM_GPIOPinTypeGPIOOutput(GPIO_PORTE_BASE, GPIO_PIN_7);
  ROM_GPIOPinWrite(GPIO_PORTE_BASE, GPIO_PIN_7, 0);
  ROM_GPIOPinTypeGPIOOutput(GPIO_PORTP_BASE, GPIO_PIN_7);
  ROM_GPIOPinWrite(GPIO_PORTP_BASE, GPIO_PIN_7, 0);
  ROM_GPIOPinTypeGPIOOutput(GPIO_PORTT_BASE, GPIO_PIN_2 | GPIO_PIN_3);
  ROM_GPIOPinWrite(GPIO_PORTT_BASE, GPIO_PIN_2 | GPIO_PIN_3, 0);

  //
  // PF0/PF4-5/PH4/PQ0-2 are used for the SPI flash (on-board and SD card).
  // PH4 selects the SD card and PQ1 selects the on-board SPI flash.
  //
  ROM_GPIOPinConfigure(GPIO_PF0_SSI3XDAT1);
  ROM_GPIOPinConfigure(GPIO_PF4_SSI3XDAT2);
  ROM_GPIOPinConfigure(GPIO_PF5_SSI3XDAT3);
  ROM_GPIOPinConfigure(GPIO_PQ0_SSI3CLK);
  ROM_GPIOPinConfigure(GPIO_PQ2_SSI3XDAT0);
  ROM_GPIOPinTypeSSI(GPIO_PORTF_BASE, GPIO_PIN_0 | GPIO_PIN_4 | GPIO_PIN_5);
  ROM_GPIOPinTypeGPIOOutput(GPIO_PORTH_BASE, GPIO_PIN_4);
  ROM_GPIOPinWrite(GPIO_PORTH_BASE, GPIO_PIN_4, GPIO_PIN_4);
  ROM_GPIOPinTypeSSI(GPIO_PORTQ_BASE, GPIO_PIN_0 | GPIO_PIN_2);
  ROM_GPIOPinTypeGPIOOutput(GPIO_PORTQ_BASE, GPIO_PIN_1);
  ROM_GPIOPinWrite(GPIO_PORTQ_BASE, GPIO_PIN_1, GPIO_PIN_1);

  //
  // PF1/PK4/PK6 are used for Ethernet LEDs.
  //
  /**///TBD: Not required???
  ROM_GPIOPinConfigure(GPIO_PF1_EN0LED2);
  ROM_GPIOPinConfigure(GPIO_PK4_EN0LED0);
  ROM_GPIOPinConfigure(GPIO_PK6_EN0LED1);
  _GPIOPinTypeEthernetLED(GPIO_PORTF_BASE, GPIO_PIN_1);
  _GPIOPinTypeEthernetLED(GPIO_PORTK_BASE, GPIO_PIN_4);
  _GPIOPinTypeEthernetLED(GPIO_PORTK_BASE, GPIO_PIN_6);

  //
  // PF6-7/PJ6/PS4-5/PR0-7 are used for the LCD.
  //
  ROM_GPIOPinConfigure(GPIO_PF7_LCDDATA02);
  ROM_GPIOPinConfigure(GPIO_PJ6_LCDAC);
  ROM_GPIOPinConfigure(GPIO_PR0_LCDCP);
  ROM_GPIOPinConfigure(GPIO_PR1_LCDFP);
  ROM_GPIOPinConfigure(GPIO_PR2_LCDLP);
  ROM_GPIOPinConfigure(GPIO_PR3_LCDDATA03);
  ROM_GPIOPinConfigure(GPIO_PR4_LCDDATA00);
  ROM_GPIOPinConfigure(GPIO_PR5_LCDDATA01);
  ROM_GPIOPinConfigure(GPIO_PR6_LCDDATA04);
  ROM_GPIOPinConfigure(GPIO_PR7_LCDDATA05);
  ROM_GPIOPinConfigure(GPIO_PS4_LCDDATA06);
  ROM_GPIOPinConfigure(GPIO_PS5_LCDDATA07);
  ROM_GPIOPinTypeGPIOOutput(GPIO_PORTF_BASE, GPIO_PIN_6);
  ROM_GPIOPinWrite(GPIO_PORTF_BASE, GPIO_PIN_6, GPIO_PIN_6);
  ROM_GPIOPinTypeLCD(GPIO_PORTF_BASE, GPIO_PIN_7);
  ROM_GPIOPinTypeLCD(GPIO_PORTJ_BASE, GPIO_PIN_6);
  ROM_GPIOPinTypeLCD(GPIO_PORTR_BASE,
                     (GPIO_PIN_0 | GPIO_PIN_1 | GPIO_PIN_2 | GPIO_PIN_3 |
                      GPIO_PIN_4 | GPIO_PIN_5 | GPIO_PIN_6 | GPIO_PIN_7));
  ROM_GPIOPinTypeLCD(GPIO_PORTS_BASE, GPIO_PIN_4 | GPIO_PIN_5);

  //
  // PQ7 is used for the user LED.
  //
  ROM_GPIOPinTypeGPIOOutput(GPIO_PORTQ_BASE, GPIO_PIN_7);
  ROM_GPIOPinWrite(GPIO_PORTQ_BASE, GPIO_PIN_7, 0);
}

/*********************************************************************
*
*       _SysCtlPeripheralEnable
*/
static void _SysCtlPeripheralEnable(uint32_t ui32Peripheral) {
    //
    // Check the arguments.
    //
    ASSERT(_SysCtlPeripheralValid(ui32Peripheral));

    //
    // Enable this peripheral.
    //
    HWREGBITW(SYSCTL_RCGCBASE + ((ui32Peripheral & 0xff00) >> 8),
              ui32Peripheral & 0xff) = 1;
}

/*********************************************************************
*
*       _GPIOPinWrite
*/
static void _GPIOPinWrite(uint32_t ui32Port, uint8_t ui8Pins, uint8_t ui8Val) {
    //
    // Check the arguments.
    //
    ASSERT(_GPIOBaseValid(ui32Port));

    //
    // Write the pins.
    //
//    HWREG(ui32Port + (GPIO_O_DATA + (ui8Pins << 2))) = ui8Val;
ROM_GPIOPinWrite(ui32Port, ui8Pins, ui8Val);
}

/*********************************************************************
*
*       _LCDModeSet
*/
static uint32_t _LCDModeSet(uint32_t ui32Base, uint8_t ui8Mode, uint32_t ui32PixClk, uint32_t ui32SysClk) {
    uint32_t ui32Div;

    //
    // Sanity check parameters.
    //
    ASSERT(ui32Base == LCD0_BASE);
    ASSERT((ui8Mode & ~(LCD_MODE_RASTER | LCD_MODE_LIDD |
                        LCD_MODE_AUTO_UFLOW_RESTART)) == 0);

    //
    // Enable clocks to the LCD controller submodules.
    //
    HWREG(ui32Base + LCD_O_CLKEN) = (LCD_CLKEN_DMA | LCD_CLKEN_CORE |
                                     LCD_CLKEN_LIDD);

    //
    // Determine the clock divider to use to get as close as possible to the
    // desired pixel clock.  Note that we set the division up so that we
    // round the divisor up and ensure that the clock used is never faster
    // than the requested rate.
    //
    ui32Div = (ui32SysClk + (ui32PixClk - 1)) / ui32PixClk;

    //
    // Check that the calculated value is valid.
    //
    ASSERT(ui32Div);
    ASSERT(ui32Div < 256);
    ASSERT(!((ui8Mode & LCD_MODE_RASTER) && (ui32Div < 2)));

    //
    // Write the LCDCTL register to set the mode.
    //
    HWREG(ui32Base + LCD_O_CTL) = (uint32_t)ui8Mode |
                                  (ui32Div << LCD_CTL_CLKDIV_S);

    //
    // Return the selected clock rate.  Finding ui32Div set to 0 should not
    // happen unless someone passed pathological arguments and builds without
    // the ASSERTS, but we guard against it just in case.
    //
    return(ui32Div ? (ui32SysClk / ui32Div) : ui32SysClk);
}

/*********************************************************************
*
*       _LCDDMAConfigSet
*/
static void _LCDDMAConfigSet(uint32_t ui32Base, uint32_t ui32Config) {
    //
    // Sanity check parameters.
    //
    ASSERT(ui32Base == LCD0_BASE);
    ASSERT(!(ui32Config & ~(LCD_DMACTL_DMAMSTRP_M | LCD_DMACTL_FIFORDY_M |
                            LCD_DMACTL_BURSTSZ_M | LCD_DMACTL_BYTESWAP |
                            LCD_DMACTL_BIGDEND | LCD_DMACTL_FMODE)));

    //
    // Write the DMA control register.
    //
    HWREG(ui32Base + LCD_O_DMACTL) = ui32Config;
}

/*********************************************************************
*
*       _LCDIDDConfigSet
*/
static void _LCDIDDConfigSet(uint32_t ui32Base, uint32_t ui32Config) {
    //
    // Sanity check parameters.
    //
    ASSERT(ui32Base == LCD0_BASE);
    ASSERT(!(ui32Config & ~(LIDD_CONFIG_SYNC_MPU68 | LIDD_CONFIG_ASYNC_MPU68 |
                            LIDD_CONFIG_SYNC_MPU80 | LIDD_CONFIG_ASYNC_MPU80 |
                            LIDD_CONFIG_ASYNC_HITACHI |
                            LIDD_CONFIG_INVERT_ALE |
                            LIDD_CONFIG_INVERT_RS_EN |
                            LIDD_CONFIG_INVERT_WS_DIR |
                            LIDD_CONFIG_INVERT_CS0 | LIDD_CONFIG_INVERT_CS1)));

    //
    // Write the LIDD Control Register.
    //
    HWREG(ui32Base + LCD_O_LIDDCTL) = ui32Config;
}

/*********************************************************************
*
*       _LCDIDDTimingSet
*/
static void _LCDIDDTimingSet(uint32_t ui32Base, uint32_t ui32CS, const tLCDIDDTiming *pTiming) {
    uint32_t ui32Val;

    //
    // Sanity check parameters.
    //
    ASSERT(ui32Base == LCD0_BASE);
    ASSERT((ui32CS == 0) || (ui32CS == 1));
    ASSERT(pTiming);
    ASSERT(pTiming->ui8WSSetup < 32);
    ASSERT(pTiming->ui8WSDuration && (pTiming->ui8WSDuration < 64));
    ASSERT(pTiming->ui8WSHold && (pTiming->ui8WSHold < 16));
    ASSERT(pTiming->ui8RSSetup < 32);
    ASSERT(pTiming->ui8RSDuration && (pTiming->ui8RSDuration < 64));
    ASSERT(pTiming->ui8RSHold && (pTiming->ui8RSHold < 16));
    ASSERT(pTiming->ui8DelayCycles && (pTiming->ui8DelayCycles < 5));

    //
    // Convert the timings provided into a value ready for the register.
    //
    ui32Val =
        (((uint32_t)(pTiming->ui8WSSetup) << LCD_LIDDCS0CFG_WRSU_S) |
         ((uint32_t)(pTiming->ui8WSDuration) << LCD_LIDDCS0CFG_WRDUR_S) |
         ((uint32_t)(pTiming->ui8WSHold) << LCD_LIDDCS0CFG_WRHOLD_S) |
         ((uint32_t)(pTiming->ui8RSSetup) << LCD_LIDDCS0CFG_RDSU_S) |
         ((uint32_t)(pTiming->ui8RSDuration) << LCD_LIDDCS0CFG_RDDUR_S) |
         ((uint32_t)(pTiming->ui8RSHold) << LCD_LIDDCS0CFG_RDHOLD_S) |
         ((uint32_t)(pTiming->ui8DelayCycles - 1) << LCD_LIDDCS0CFG_GAP_S));

    //
    // Write the appropriate LCD LIDD CS configuration register.
    //
    if(!ui32CS)
    {
        HWREG(ui32Base + LCD_O_LIDDCS0CFG) = ui32Val;
    }
    else
    {
        HWREG(ui32Base + LCD_O_LIDDCS1CFG) = ui32Val;
    }
}

/*********************************************************************
*
*       _TouchScreenCallback
*/
static int32_t _TouchScreenCallback(uint32_t Pressed,  int32_t  x, int32_t  y) {
  GUI_PID_STATE State = {0};

  State.x = x;
  State.y = y;
  State.Pressed = Pressed;
  GUI_PID_StoreState(&State);
  return 0;
}


/*********************************************************************
*
*       Public code
*
**********************************************************************
*/
/*********************************************************************
*
*       HW_Init
*/
void HW_Init(void) {
  tLCDIDDTiming sTimings;

  //
  // Configure device pins
  //
  _Pinout();
  //
  // Enable the LCD controller.
  //
  _SysCtlPeripheralEnable(SYSCTL_PERIPH_LCD0);
  //
  // Assert the LCD reset signal.
  //
  _GPIOPinWrite(GPIO_PORTF_BASE, GPIO_PIN_6, 0);
  //
  // Delay for 50ms.
  //
  GUI_X_Delay(100);
  //
  // Deassert the LCD reset signal.
  //
  _GPIOPinWrite(GPIO_PORTF_BASE, GPIO_PIN_6, GPIO_PIN_6);
  //
  // Delay for 50ms.
  //
  GUI_X_Delay(50);
  //
  // Configure the LCD controller for LIDD-mode operation.
  //
  _LCDModeSet(LCD0_BASE, LCD_MODE_LIDD, SYS_CLK, SYS_CLK);
  //
  // Configure DMA-related parameters.
  //
  _LCDDMAConfigSet(LCD0_BASE, LCD_DMA_BURST_4);
  //
  // Set control signal parameters and polarities.
  //
  _LCDIDDConfigSet(LCD0_BASE, LIDD_CONFIG_ASYNC_MPU80);
  //
  // Set the LIDD interface timings for the Kentec display.  Note that the
  // inter-transaction delay is set at at 50nS to match the write case.
  // Software needs to ensure that it delays at least 450nS more between each
  // read or the read timings will be violated.
  //
  sTimings.ui8WSSetup     = CYCLES_FROM_TIME_NS(SYS_CLK, 5);
  sTimings.ui8WSDuration  = CYCLES_FROM_TIME_NS(SYS_CLK, 40);
  sTimings.ui8WSHold      = CYCLES_FROM_TIME_NS(SYS_CLK, 5);
  sTimings.ui8RSSetup     = CYCLES_FROM_TIME_NS(SYS_CLK, 0);
  sTimings.ui8RSDuration  = CYCLES_FROM_TIME_NS(SYS_CLK, 500);
  sTimings.ui8RSHold      = CYCLES_FROM_TIME_NS(SYS_CLK, 100);
  sTimings.ui8DelayCycles = CYCLES_FROM_TIME_NS(SYS_CLK, 50);
  _LCDIDDTimingSet(LCD0_BASE, 0, &sTimings);
  //
  // Initialize the touch screen driver and have it route its messages to emWin
  //
  TouchScreenInit(SYS_CLK);
  TouchScreenCallbackSet(_TouchScreenCallback);
}

/*************************** End of file ****************************/

