/*********************************************************************
*                SEGGER MICROCONTROLLER GmbH & Co. KG                *
*        Solutions for real time microcontroller applications        *
**********************************************************************
*                                                                    *
*        (c) 2003-2014     SEGGER Microcontroller GmbH & Co KG       *
*                                                                    *
*        Internet: www.segger.com    Support:  support@segger.com    *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File        : LCDConf.c
Purpose     : Display controller configuration (single layer)
---------------------------END-OF-HEADER------------------------------
*/

#include "HW_TM4C129.h"

#include "GUI.h"
#include "GUIDRV_FlexColor.h"

/*********************************************************************
*
*       Layer configuration (to be modified)
*
**********************************************************************
*/
//
// Physical display size
//
#define XSIZE_PHYS 320
#define YSIZE_PHYS 240

//
// Color conversion
//
#define COLOR_CONVERSION GUICC_M565

//
// Display driver
//
#define DISPLAY_DRIVER GUIDRV_FLEXCOLOR

//
// Buffers / VScreens
//
#define NUM_BUFFERS  1 // Number of multiple buffers to be used
#define NUM_VSCREENS 1 // Number of virtual screens to be used

//
// Display orientation
//
#define DISPLAY_ORIENTATION 0

/*********************************************************************
*
*       Display controller register definitions
*
**********************************************************************
*/
#define SSD2119_DEVICE_CODE_READ_REG  0x00
#define SSD2119_OSC_START_REG         0x00
#define SSD2119_OUTPUT_CTRL_REG       0x01
#define SSD2119_LCD_DRIVE_AC_CTRL_REG 0x02
#define SSD2119_PWR_CTRL_1_REG        0x03
#define SSD2119_DISPLAY_CTRL_REG      0x07
#define SSD2119_FRAME_CYCLE_CTRL_REG  0x0b
#define SSD2119_PWR_CTRL_2_REG        0x0c
#define SSD2119_PWR_CTRL_3_REG        0x0d
#define SSD2119_PWR_CTRL_4_REG        0x0e
#define SSD2119_GATE_SCAN_START_REG   0x0f
#define SSD2119_SLEEP_MODE_1_REG      0x10
#define SSD2119_ENTRY_MODE_REG        0x11
#define SSD2119_SLEEP_MODE_2_REG      0x12
#define SSD2119_GEN_IF_CTRL_REG       0x15
#define SSD2119_PWR_CTRL_5_REG        0x1e
#define SSD2119_RAM_DATA_REG          0x22
#define SSD2119_FRAME_FREQ_REG        0x25
#define SSD2119_ANALOG_SET_REG        0x26
#define SSD2119_VCOM_OTP_1_REG        0x28
#define SSD2119_VCOM_OTP_2_REG        0x29
#define SSD2119_GAMMA_CTRL_1_REG      0x30
#define SSD2119_GAMMA_CTRL_2_REG      0x31
#define SSD2119_GAMMA_CTRL_3_REG      0x32
#define SSD2119_GAMMA_CTRL_4_REG      0x33
#define SSD2119_GAMMA_CTRL_5_REG      0x34
#define SSD2119_GAMMA_CTRL_6_REG      0x35
#define SSD2119_GAMMA_CTRL_7_REG      0x36
#define SSD2119_GAMMA_CTRL_8_REG      0x37
#define SSD2119_GAMMA_CTRL_9_REG      0x3a
#define SSD2119_GAMMA_CTRL_10_REG     0x3b
#define SSD2119_V_RAM_POS_REG         0x44
#define SSD2119_H_RAM_START_REG       0x45
#define SSD2119_H_RAM_END_REG         0x46
#define SSD2119_X_RAM_ADDR_REG        0x4e
#define SSD2119_Y_RAM_ADDR_REG        0x4f

#define ENTRY_MODE_DEFAULT            0x6830
#define MAKE_ENTRY_MODE(x)            ((ENTRY_MODE_DEFAULT & 0xff00) | (x))

/*********************************************************************
*
*       Configuration checking
*
**********************************************************************
*/
#ifndef   VRAM_ADDR
  #define VRAM_ADDR 0 // TBD by customer: This has to be the frame buffer start address
#endif
#ifndef   XSIZE_PHYS
  #error Physical X size of display is not defined!
#endif
#ifndef   YSIZE_PHYS
  #error Physical Y size of display is not defined!
#endif
#ifndef   COLOR_CONVERSION
  #error Color conversion not defined!
#endif
#ifndef   DISPLAY_DRIVER
  #error No display driver defined!
#endif
#ifndef   NUM_VSCREENS
  #define NUM_VSCREENS 1
#else
  #if (NUM_VSCREENS <= 0)
    #error At least one screeen needs to be defined!
  #endif
#endif
#if (NUM_VSCREENS > 1) && (NUM_BUFFERS > 1)
  #error Virtual screens and multiple buffers are not allowed!
#endif

/*********************************************************************
*
*       _WriteCommand8
*/
static inline void _WriteCommand8(U8 Cmd) {
  HWREG(LCD0_BASE + LCD_O_LIDDCS0ADDR) = Cmd;
}

/*********************************************************************
*
*       _WriteData8
*/
static inline void _WriteData8(U8 Data) {
  HWREG(LCD0_BASE + LCD_O_LIDDCS0DATA) = Data;
}

/*********************************************************************
*
*       _WriteDataM8
*/
static inline void _WriteDataM8(U8 * pData, int NumItems) {
  do {
    HWREG(LCD0_BASE + LCD_O_LIDDCS0DATA) = *pData++;
  } while (--NumItems);
}

/*********************************************************************
*
*       _ReadData8
*/
static inline U8 _ReadData8(void) {
  return (U8)HWREG(LCD0_BASE + LCD_O_LIDDCS0DATA);
}

/*********************************************************************
*
*       _ReadDataM8
*/
static inline void _ReadDataM8(U8 * pData, int NumItems) {
  do {
    *pData++ = (U8)HWREG(LCD0_BASE + LCD_O_LIDDCS0DATA);
  } while (--NumItems);
}

/*********************************************************************
*
*       _WriteData16
*/
static inline void _WriteData16(U16 Data) {
  HWREG(LCD0_BASE + LCD_O_LIDDCS0DATA) = Data >> 8;
  HWREG(LCD0_BASE + LCD_O_LIDDCS0DATA) = Data & 0xFF;
}

/*********************************************************************
*
*       _InitController
*/
static void _InitController(void) {
  //
  // Initialize ports, LIDD interface and management of touch screen
  //
  HW_Init();
  //
  // Enter sleep mode (if not already there).
  //
  _WriteCommand8(SSD2119_SLEEP_MODE_1_REG);
  _WriteData16(0x0001);
  //
  // Set initial power parameters.
  //
  _WriteCommand8(SSD2119_PWR_CTRL_5_REG);
  _WriteData16(0x00b2);
  _WriteCommand8(SSD2119_VCOM_OTP_1_REG);
  _WriteData16(0x0006);
  //
  // Start the oscillator.
  //
  _WriteCommand8(SSD2119_OSC_START_REG);
  _WriteData16(0x0001);
  //
  // Set pixel format and basic display orientation (scanning direction).
  //
  _WriteCommand8(SSD2119_OUTPUT_CTRL_REG);
  _WriteData16(0x30ef);
  _WriteCommand8(SSD2119_LCD_DRIVE_AC_CTRL_REG);
  _WriteData16(0x0600);
  //
  // Exit sleep mode.
  //
  _WriteCommand8(SSD2119_SLEEP_MODE_1_REG);
  _WriteData16(0x0000);
  //
  // Delay 30mS
  //
  GUI_X_Delay(30);
  //
  // Configure pixel color format and MCU interface parameters.
  //
  _WriteCommand8(SSD2119_ENTRY_MODE_REG);
  _WriteData16(ENTRY_MODE_DEFAULT);
  //
  // Set analog parameters.
  //
  _WriteCommand8(SSD2119_SLEEP_MODE_2_REG);
  _WriteData16(0x0999);
  _WriteCommand8(SSD2119_ANALOG_SET_REG);
  _WriteData16(0x3800);
  //
  // Enable the display.
  //
  _WriteCommand8(SSD2119_DISPLAY_CTRL_REG);
  _WriteData16(0x0033);
  //
  // Set VCIX2 voltage to 6.1V.
  //
  _WriteCommand8(SSD2119_PWR_CTRL_2_REG);
  _WriteData16(0x0005);
  //
  // Configure gamma correction.
  //
  _WriteCommand8(SSD2119_GAMMA_CTRL_1_REG);
  _WriteData16(0x0000);
  _WriteCommand8(SSD2119_GAMMA_CTRL_2_REG);
  _WriteData16(0x0303);
  _WriteCommand8(SSD2119_GAMMA_CTRL_3_REG);
  _WriteData16(0x0407);
  _WriteCommand8(SSD2119_GAMMA_CTRL_4_REG);
  _WriteData16(0x0301);
  _WriteCommand8(SSD2119_GAMMA_CTRL_5_REG);
  _WriteData16(0x0301);
  _WriteCommand8(SSD2119_GAMMA_CTRL_6_REG);
  _WriteData16(0x0403);
  _WriteCommand8(SSD2119_GAMMA_CTRL_7_REG);
  _WriteData16(0x0707);
  _WriteCommand8(SSD2119_GAMMA_CTRL_8_REG);
  _WriteData16(0x0400);
  _WriteCommand8(SSD2119_GAMMA_CTRL_9_REG);
  _WriteData16(0x0a00);
  _WriteCommand8(SSD2119_GAMMA_CTRL_10_REG);
  _WriteData16(0x1000);
  //
  // Configure Vlcd63 and VCOMl.
  //
  _WriteCommand8(SSD2119_PWR_CTRL_3_REG);
  _WriteData16(0x000a);
  _WriteCommand8(SSD2119_PWR_CTRL_4_REG);
  _WriteData16(0x2e00);
}

/*********************************************************************
*
*       Public code
*
**********************************************************************
*/
/*********************************************************************
*
*       LCD_X_Config
*
* Purpose:
*   Called during the initialization process in order to set up the
*   display driver configuration.
*   
*/
void LCD_X_Config(void) {
  GUI_DEVICE * pDevice;
  CONFIG_FLEXCOLOR Config = {0};
  GUI_PORT_API PortAPI = {0};

  //
  // Set display driver and color conversion for 1st layer
  //
  pDevice = GUI_DEVICE_CreateAndLink(DISPLAY_DRIVER, COLOR_CONVERSION, 0, 0);
  //
  // Common display driver configuration
  //
  LCD_SetSizeEx (0, XSIZE_PHYS, YSIZE_PHYS);
  LCD_SetVSizeEx(0, XSIZE_PHYS, YSIZE_PHYS * NUM_VSCREENS);
  //
  // Configuration of GUIDRV_FLEX_COLOR
  //
  GUIDRV_FlexColor_Config(pDevice, &Config);
  //
  // Orientation
  //
  Config.Orientation   = DISPLAY_ORIENTATION;
  Config.RegEntryMode  = ENTRY_MODE_DEFAULT;
  GUIDRV_FlexColor_Config(pDevice, &Config);
  //
  // Set controller and operation mode
  //
  PortAPI.pfWrite8_A0  = _WriteCommand8;
  PortAPI.pfWrite8_A1  = _WriteData8;
  PortAPI.pfWriteM8_A1 = _WriteDataM8;
  PortAPI.pfRead8_A1   = _ReadData8;
  PortAPI.pfReadM8_A1  = _ReadDataM8;
  GUIDRV_FlexColor_SetFunc(pDevice, &PortAPI, GUIDRV_FLEXCOLOR_F66714, GUIDRV_FLEXCOLOR_M16C0B8);
}

/*********************************************************************
*
*       LCD_X_DisplayDriver
*
* Purpose:
*   This function is called by the display driver for several purposes.
*   To support the according task the routine needs to be adapted to
*   the display controller. Please note that the commands marked with
*   'optional' are not cogently required and should only be adapted if 
*   the display controller supports these features.
*
* Parameter:
*   LayerIndex - Index of layer to be configured
*   Cmd        - Please refer to the details in the switch statement below
*   pData      - Pointer to a LCD_X_DATA structure
*
* Return Value:
*   < -1 - Error
*     -1 - Command not handled
*      0 - Ok
*/
int LCD_X_DisplayDriver(unsigned LayerIndex, unsigned Cmd, void * pData) {
  int r;

  switch (Cmd) {
  case LCD_X_INITCONTROLLER:
    _InitController();
    return 0;
  default:
    r = -1;
  }
  return r;
}

/*************************** End of file ****************************/
