/*********************************************************************
*                SEGGER MICROCONTROLLER GmbH & Co. KG                *
*        Solutions for real time microcontroller applications        *
**********************************************************************
*                                                                    *
*        (c) 2003-2014     SEGGER Microcontroller GmbH & Co KG       *
*                                                                    *
*        Internet: www.segger.com    Support:  support@segger.com    *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : OS_Priv.h
Purpose : CPU and compiler specifics for CM3/CM4/CM4F and GNU,
          OS internal
--------  END-OF-HEADER  ---------------------------------------------
*/

#ifndef OS_PRIVATE_H                    /* Avoid multiple inclusion */
#define OS_PRIVATE_H

/*********************************************************************
*
*       CPU and Library Name
*
**********************************************************************
*/
#if (defined (__VFP_FP__) && !defined (__SOFTFP__))
  #define OS_CPU         "emIDE CM4F"
  #define OS_CPU_PREFIX  "v7v"
  #define OS_CPU_HAS_VFP  (1)
#else
  #define OS_CPU         "emIDE CM3/CM4"
  #define OS_CPU_PREFIX  "v7"
#endif

#ifdef __ARMEB__
  #define OS_ENDIAN_PREFIX "B"
#else 
  #ifdef __ARMEL__
    #define OS_ENDIAN_PREFIX "L"
  #else
    #error "Your GCC does not support __ARMEB__ or __ARMEL__."
  #endif
#endif


#define OS_MODEL OS_CPU_PREFIX OS_ENDIAN_PREFIX

#define OS_PORT_VERSION 38800      // Version number of tested version

/*********************************************************************
*
*       Configuration
*
**********************************************************************
*/

#define OS_STACK_ALIGN            (8)   // Ensure task stack alignment to 8 byte boundaries
#define OS_GLOBAL_IS_PARA         (1)   // OS_Global is given as parameter to OS_ChangeTask() and OS_Switch()
#define OS_GLOBAL_IS_RETURNED     (1)   // OS_Global is returned from OS_ChangeTask() (Otimization for OS_Switch())

#define VALUE_2PTR(x)             ((int)x)

/****** End of configurable settings ********************************/

#ifndef   OS_CPU_HAS_VFP
  #define OS_CPU_HAS_VFP          (0)
#endif

/*********************************************************************
*
*       Register structure, Chip specifics CM3
*
**********************************************************************
*/

#ifdef __cplusplus
  extern "C" {
#endif
  void OS_InitMode        (void);       
  int  OS_ARM_ISRCheckPrio(int ISRIndex);
#ifdef __cplusplus
  }
#endif

#define OS_INIT_STACK_INFO()  OS_InitMode()

/****** Default Floating point control register, set by OS_InitMode */

#define FPDSCR                      (*(volatile OS_U32*) (0xE000EF3C)) 

/*********************************************************************
*
*       Regsiter structure (context) on task stack
*/
#if OS_CPU_HAS_VFP
  // Cortex M4F or Cortex M4 with VFP selected
typedef struct {
  OS_REGS_BASE Regs;            // Base registers are multiples of 8 bytes
  OS_U32       DummyAlignTo8;   // Fill up 4 bytes to keep 8 byte stack alignement when task is started. Has to be popped in OS_StartTask 
  OS_U32       PC_Task;            
} OS_REGS_INIT;

#else
  // Cortex M0, M3, M4 without VFP
typedef struct {
  OS_REGS_BASE Regs;
  OS_U32       PC_Task;
} OS_REGS_INIT;

#endif   // OS_CPU_HAS_VFP

/*********************************************************************
*
*       Register initialization, called from OS_CreateTask()
*/
#if OS_CPU_HAS_VFP
  //
  // The Link register is part of the task context, because it flags whether extended stack context is used.
  // Initially, the task is started without extended stack context. Refer to ARM DDI 0403D Page B1-653
  //
  #define OS_INIT_REGS_VFP() (((OS_REGS_INIT*)pStack)->Regs.OS_REG_LR = 0xFFFFFFFDuL)
#else
  #define OS_INIT_REGS_VFP()
#endif

#if OS_DEBUG
  #define OS_INITREGS_DEBUG()                                           \
  ((OS_REGS_INIT*)pStack)->Regs.OS_REG_R1       = 0xCCCC0001;           \
  ((OS_REGS_INIT*)pStack)->Regs.OS_REG_R2       = 0xCCCC0002;           \
  ((OS_REGS_INIT*)pStack)->Regs.OS_REG_R3       = 0xCCCC0003;           \
  ((OS_REGS_INIT*)pStack)->Regs.OS_REG_R4       = 0xCCCC0004;           \
  ((OS_REGS_INIT*)pStack)->Regs.OS_REG_R5       = 0xCCCC0005;           \
  ((OS_REGS_INIT*)pStack)->Regs.OS_REG_R6       = 0xCCCC0006;           \
  ((OS_REGS_INIT*)pStack)->Regs.OS_REG_R7       = 0xCCCC0007;           \
  ((OS_REGS_INIT*)pStack)->Regs.OS_REG_R8       = 0xCCCC0008;           \
  ((OS_REGS_INIT*)pStack)->Regs.OS_REG_R9       = 0xCCCC0009;           \
  ((OS_REGS_INIT*)pStack)->Regs.OS_REG_R10      = 0xCCCC000A;           \
  ((OS_REGS_INIT*)pStack)->Regs.OS_REG_R11      = 0xCCCC000B;           
#else
  #define OS_INITREGS_DEBUG()
#endif

#define OS_INITREGS()                                                   \
   {                                                                    \
   ((OS_REGS_INIT*)pStack)->Regs.Counters     = 0;                      \
   ((OS_REGS_INIT*)pStack)->Regs.OS_REG_XPSR  = 0x01000000;             \
   ((OS_REGS_INIT*)pStack)->Regs.OS_REG_PC    = (OS_U32)OS_StartTask;   \
   ((OS_REGS_INIT*)pStack)->PC_Task           = (OS_U32)pRoutine;       \
   OS_INIT_REGS_VFP();                                                  \
   OS_INITREGS_DEBUG();                                                 \
   }

/*********************************************************************
*
*       Register initialization, called from OS_CreateTaskEx()
*       OS_StartTask (in assembly part) has to copy the value from R4
*       into R0 to pass it as parameter to the task
*/
#define OS_INITREGS_EX(pContext) OS_INITREGS()                          \
   ((OS_REGS_INIT*)pStack)->Regs.OS_REG_R4    = (int)(pContext);


/*********************************************************************
*
*       VFP support functions
*/
#if OS_CPU_HAS_VFP
//
// VFP Save and Restore will be called by the new function
// OS_ExtendTaskContext_TLS_VFP()
// Therefore they have to be declared here (private functions)
//
void OS_VFP_SaveRegs   (void * pStack);
void OS_VFP_RestoreRegs(const void * pStack);
#endif

/********************************************************************/

#endif                                // Avoid multiple inclusion

/****** End Of File *************************************************/
